@@ -1,5 +1,6 @@
 require 'securerandom'
 require 'bindata'
+require 'aes_key_wrap'
 
 module JSON
   class JWE
@@ -31,13 +32,15 @@ module JSON
       self.mac_key, self.encryption_key = derive_encryption_and_mac_keys
       cipher.key = encryption_key
       self.iv = cipher.random_iv # NOTE: 'iv' has to be set after 'key' for GCM
-      self.auth_data = UrlSafeBase64.encode64 header.to_json
+      self.auth_data = Base64.urlsafe_encode64 header.to_json, padding: false
       cipher.auth_data = auth_data if gcm?
       self.cipher_text = cipher.update(plain_text) + cipher.final
       self
     end
 
-    def decrypt!(private_key_or_secret)
+    def decrypt!(private_key_or_secret, algorithms = nil, encryption_methods = nil)
+      raise UnexpectedAlgorithm.new('Unexpected alg header') unless algorithms.blank? || Array(algorithms).include?(alg)
+      raise UnexpectedAlgorithm.new('Unexpected enc header') unless encryption_methods.blank? || Array(encryption_methods).include?(enc)
       self.private_key_or_secret = with_jwk_support private_key_or_secret
       cipher.decrypt
       self.content_encryption_key = decrypt_content_encryption_key
@@ -45,6 +48,8 @@ module JSON
       cipher.key = encryption_key
       cipher.iv = iv # NOTE: 'iv' has to be set after 'key' for GCM
       if gcm?
+        # https://github.com/ruby/openssl/issues/63
+        raise DecryptionFailed.new('Invalid authentication tag') if authentication_tag.length < 16
         cipher.auth_tag = authentication_tag
         cipher.auth_data = auth_data
       end
@@ -61,7 +66,7 @@ module JSON
         cipher_text,
         authentication_tag
       ].collect do |segment|
-        UrlSafeBase64.encode64 segment.to_s
+        Base64.urlsafe_encode64 segment.to_s, padding: false
       end.join('.')
     end
 
@@ -69,21 +74,21 @@ module JSON
       case options[:syntax]
       when :general
         {
-          protected:  UrlSafeBase64.encode64(header.to_json),
+          protected:  Base64.urlsafe_encode64(header.to_json, padding: false),
           recipients: [{
-            encrypted_key: UrlSafeBase64.encode64(jwe_encrypted_key)
+            encrypted_key: Base64.urlsafe_encode64(jwe_encrypted_key, padding: false)
           }],
-          iv:         UrlSafeBase64.encode64(iv),
-          ciphertext: UrlSafeBase64.encode64(cipher_text),
-          tag:        UrlSafeBase64.encode64(authentication_tag)
+          iv:         Base64.urlsafe_encode64(iv, padding: false),
+          ciphertext: Base64.urlsafe_encode64(cipher_text, padding: false),
+          tag:        Base64.urlsafe_encode64(authentication_tag, padding: false)
         }
       else
         {
-          protected:     UrlSafeBase64.encode64(header.to_json),
-          encrypted_key: UrlSafeBase64.encode64(jwe_encrypted_key),
-          iv:            UrlSafeBase64.encode64(iv),
-          ciphertext:    UrlSafeBase64.encode64(cipher_text),
-          tag:           UrlSafeBase64.encode64(authentication_tag)
+          protected:     Base64.urlsafe_encode64(header.to_json, padding: false),
+          encrypted_key: Base64.urlsafe_encode64(jwe_encrypted_key, padding: false),
+          iv:            Base64.urlsafe_encode64(iv, padding: false),
+          ciphertext:    Base64.urlsafe_encode64(cipher_text, padding: false),
+          tag:           Base64.urlsafe_encode64(authentication_tag, padding: false)
         }
       end
     end
@@ -92,10 +97,6 @@ module JSON
 
     # common
 
-    def gcm_supported?
-      RUBY_VERSION >= '2.0.0' && OpenSSL::OPENSSL_VERSION >= 'OpenSSL 1.0.1'
-    end
-
     def gcm?
       [:A128GCM, :A256GCM].include? encryption_method.try(:to_sym)
     end
@@ -109,11 +110,8 @@ module JSON
     end
 
     def cipher
-      @cipher ||= if gcm? && !gcm_supported?
-        raise UnexpectedAlgorithm.new('AEC GCM requires Ruby 2.0+ and OpenSSL 1.0.1c+')
-      else
-        OpenSSL::Cipher.new cipher_name
-      end
+      raise "#{cipher_name} isn't supported" unless OpenSSL::Cipher.ciphers.include?(cipher_name)
+      @cipher ||= OpenSSL::Cipher.new cipher_name
     end
 
     def cipher_name
@@ -165,10 +163,8 @@ module JSON
         public_key_or_secret.public_encrypt content_encryption_key
       when :'RSA-OAEP'
         public_key_or_secret.public_encrypt content_encryption_key, OpenSSL::PKey::RSA::PKCS1_OAEP_PADDING
-      when :A128KW
-        raise NotImplementedError.new('A128KW not supported yet')
-      when :A256KW
-        raise NotImplementedError.new('A256KW not supported yet')
+      when :A128KW, :A256KW
+        AESKeyWrap.wrap content_encryption_key, public_key_or_secret
       when :dir
         ''
       when :'ECDH-ES'
@@ -219,10 +215,8 @@ module JSON
         private_key_or_secret.private_decrypt jwe_encrypted_key
       when :'RSA-OAEP'
         private_key_or_secret.private_decrypt jwe_encrypted_key, OpenSSL::PKey::RSA::PKCS1_OAEP_PADDING
-      when :A128KW
-        raise NotImplementedError.new('A128KW not supported yet')
-      when :A256KW
-        raise NotImplementedError.new('A256KW not supported yet')
+      when :A128KW, :A256KW
+        AESKeyWrap.unwrap jwe_encrypted_key, private_key_or_secret
       when :dir
         private_key_or_secret
       when :'ECDH-ES'
@@ -254,21 +248,27 @@ module JSON
     end
 
     class << self
-      def decode_compact_serialized(input, private_key_or_secret)
+      def decode_compact_serialized(input, private_key_or_secret, algorithms = nil, encryption_methods = nil)
         unless input.count('.') + 1 == NUM_OF_SEGMENTS
           raise InvalidFormat.new("Invalid JWE Format. JWE should include #{NUM_OF_SEGMENTS} segments.")
         end
         jwe = new
         _header_json_, jwe.jwe_encrypted_key, jwe.iv, jwe.cipher_text, jwe.authentication_tag = input.split('.').collect do |segment|
-          UrlSafeBase64.decode64 segment
+          begin
+            Base64.urlsafe_decode64 segment
+          rescue ArgumentError
+            raise DecryptionFailed
+          end
         end
         jwe.auth_data = input.split('.').first
-        jwe.header = MultiJson.load(_header_json_).with_indifferent_access
-        jwe.decrypt! private_key_or_secret unless private_key_or_secret == :skip_decryption
+        jwe.header = JSON.parse(_header_json_).with_indifferent_access
+        unless private_key_or_secret == :skip_decryption
+          jwe.decrypt! private_key_or_secret, algorithms, encryption_methods
+        end
         jwe
       end
 
-      def decode_json_serialized(input, private_key_or_secret)
+      def decode_json_serialized(input, private_key_or_secret, algorithms = nil, encryption_methods = nil)
         input = input.with_indifferent_access
         jwe_encrypted_key = if input[:recipients].present?
           input[:recipients].first[:encrypted_key]
@@ -282,8 +282,8 @@ module JSON
           input[:ciphertext],
           input[:tag]
         ].join('.')
-        decode_compact_serialized compact_serialized, private_key_or_secret
+        decode_compact_serialized compact_serialized, private_key_or_secret, algorithms, encryption_methods
       end
     end
   end
-end
\ No newline at end of file
+end
