@@ -34,7 +34,7 @@ module JSON
       else
         raise UnknownAlgorithm.new('Unknown Digest Algorithm')
       end
-      UrlSafeBase64.encode64 digest.digest(normalize.to_json)
+      Base64.urlsafe_encode64 digest.digest(normalize.to_json), padding: false
     end
 
     def to_key
@@ -98,7 +98,7 @@ module JSON
     def to_rsa_key
       e, n, d, p, q, dp, dq, qi = [:e, :n, :d, :p, :q, :dp, :dq, :qi].collect do |key|
         if self[key]
-          OpenSSL::BN.new UrlSafeBase64.decode64(self[key]), 2
+          OpenSSL::BN.new Base64.urlsafe_decode64(self[key]), 2
         end
       end
       key = OpenSSL::PKey::RSA.new
@@ -132,14 +132,14 @@ module JSON
       end
       x, y, d = [:x, :y, :d].collect do |key|
         if self[key]
-          OpenSSL::BN.new UrlSafeBase64.decode64(self[key]), 2
+          Base64.urlsafe_decode64(self[key])
         end
       end
       key = OpenSSL::PKey::EC.new curve_name
-      key.private_key = d if d
+      key.private_key = OpenSSL::BN.new(d, 2) if d
       key.public_key = OpenSSL::PKey::EC::Point.new(
         OpenSSL::PKey::EC::Group.new(curve_name),
-        OpenSSL::BN.new(['04' + x.to_s(16) + y.to_s(16)].pack('H*'), 2)
+        OpenSSL::BN.new(['04' + x.unpack('H*').first + y.unpack('H*').first].pack('H*'), 2)
       )
       key
     end
