@@ -13,17 +13,20 @@ module JSON
     end
 
     def sign!(private_key_or_secret)
+      self.alg = autodetected_algorithm_from(private_key_or_secret) if algorithm == :autodetect
       self.signature = sign signature_base_string, private_key_or_secret
       self
     end
 
-    def verify!(public_key_or_secret)
+    def verify!(public_key_or_secret, algorithms = nil)
       if alg.try(:to_sym) == :none
         raise UnexpectedAlgorithm if public_key_or_secret
         signature == '' or raise VerificationFailed
-      else
+      elsif algorithms.blank? || Array(algorithms).include?(alg.try(:to_sym))
         public_key_or_secret && valid?(public_key_or_secret) or
         raise VerificationFailed
+      else
+        raise UnexpectedAlgorithm.new('Unexpected alg header')
       end
     end
 
@@ -50,16 +53,51 @@ module JSON
       [:RS256, :RS384, :RS512].include? algorithm.try(:to_sym)
     end
 
+    def rsa_pss?
+      if [:PS256, :PS384, :PS512].include? algorithm.try(:to_sym)
+        if OpenSSL::VERSION < '2.1.0'
+          raise "#{alg} isn't supported. OpenSSL gem v2.1.0+ is required to use #{alg}."
+        else
+          true
+        end
+      else
+        false
+      end
+    end
+
     def ecdsa?
       [:ES256, :ES384, :ES512].include? algorithm.try(:to_sym)
     end
 
+    def autodetected_algorithm_from(private_key_or_secret)
+      private_key_or_secret = with_jwk_support private_key_or_secret
+      case private_key_or_secret
+      when String
+        :HS256
+      when OpenSSL::PKey::RSA
+        :RS256
+      when OpenSSL::PKey::EC
+        case private_key_or_secret.group.curve_name
+        when 'prime256v1'
+          :ES256
+        when 'secp384r1'
+          :ES384
+        when 'secp521r1'
+          :ES512
+        else
+          raise UnknownAlgorithm.new('Unknown EC Curve')
+        end
+      else
+        raise UnexpectedAlgorithm.new('Signature algorithm auto-detection failed')
+      end
+    end
+
     def signature_base_string
       @signature_base_string ||= [
         header.to_json,
         self.to_json
       ].collect do |segment|
-        UrlSafeBase64.encode64 segment
+        Base64.urlsafe_encode64 segment, padding: false
       end.join('.')
     end
 
@@ -72,6 +110,9 @@ module JSON
       when rsa?
         private_key = private_key_or_secret
         private_key.sign digest, signature_base_string
+      when rsa_pss?
+        private_key = private_key_or_secret
+        private_key.sign_pss digest, signature_base_string, salt_length: :digest, mgf1_hash: digest
       when ecdsa?
         private_key = private_key_or_secret
         verify_ecdsa_group! private_key
@@ -92,6 +133,9 @@ module JSON
       when rsa?
         public_key = public_key_or_secret
         public_key.verify digest, signature, signature_base_string
+      when rsa_pss?
+        public_key = public_key_or_secret
+        public_key.verify_pss digest, signature, signature_base_string, salt_length: :digest, mgf1_hash: digest
       when ecdsa?
         public_key = public_key_or_secret
         verify_ecdsa_group! public_key
@@ -132,25 +176,25 @@ module JSON
     end
 
     class << self
-      def decode_compact_serialized(input, public_key_or_secret)
+      def decode_compact_serialized(input, public_key_or_secret, algorithms = nil)
         unless input.count('.') + 1 == NUM_OF_SEGMENTS
           raise InvalidFormat.new("Invalid JWS Format. JWS should include #{NUM_OF_SEGMENTS} segments.")
         end
         header, claims, signature = input.split('.', JWS::NUM_OF_SEGMENTS).collect do |segment|
-          UrlSafeBase64.decode64 segment.to_s
+          Base64.urlsafe_decode64 segment.to_s
         end
         header, claims = [header, claims].collect do |json|
-          MultiJson.load(json).with_indifferent_access
+          JSON.parse(json).with_indifferent_access
         end
         jws = new claims
         jws.header = header
         jws.signature = signature
         jws.signature_base_string = input.split('.')[0, JWS::NUM_OF_SEGMENTS - 1].join('.')
-        jws.verify! public_key_or_secret unless public_key_or_secret == :skip_verification
+        jws.verify! public_key_or_secret, algorithms unless public_key_or_secret == :skip_verification
         jws
       end
 
-      def decode_json_serialized(input, public_key_or_secret)
+      def decode_json_serialized(input, public_key_or_secret, algorithms = nil)
         input = input.with_indifferent_access
         header, payload, signature = if input[:signatures].present?
           [
@@ -166,7 +210,7 @@ module JSON
           end
         end
         compact_serialized = [header, payload, signature].join('.')
-        decode_compact_serialized compact_serialized, public_key_or_secret
+        decode_compact_serialized compact_serialized, public_key_or_secret, algorithms
       end
     end
   end
