@@ -1,6 +1,5 @@
 require 'openssl'
-require 'url_safe_base64'
-require 'multi_json'
+require 'base64'
 require 'active_support'
 require 'active_support/core_ext'
 require 'json/jose'
@@ -27,13 +26,6 @@ module JSON
     end
 
     def sign(private_key_or_secret, algorithm = :autodetect)
-      if algorithm == :autodetect
-        # NOTE:
-        #  I'd like to make :RS256 default.
-        #  However, by histrical reasons, :HS256 was default.
-        #  This code is needed to keep legacy behavior.
-        algorithm = private_key_or_secret.is_a?(String) ? :HS256 : :RS256
-      end
       jws = JWS.new self
       jws.kid ||= private_key_or_secret[:kid] if private_key_or_secret.is_a? JSON::JWK
       jws.alg = algorithm
@@ -54,7 +46,7 @@ module JSON
         self.to_json,
         signature
       ].collect do |segment|
-        UrlSafeBase64.encode64 segment.to_s
+        Base64.urlsafe_encode64 segment.to_s, padding: false
       end.join('.')
     end
 
@@ -62,45 +54,56 @@ module JSON
       case options[:syntax]
       when :general
         {
-          payload: UrlSafeBase64.encode64(self.to_json),
+          payload: Base64.urlsafe_encode64(self.to_json, padding: false),
           signatures: [{
-            protected: UrlSafeBase64.encode64(header.to_json),
-            signature: UrlSafeBase64.encode64(signature.to_s)
+            protected: Base64.urlsafe_encode64(header.to_json, padding: false),
+            signature: Base64.urlsafe_encode64(signature.to_s, padding: false)
           }]
         }
       when :flattened
         {
-          protected: UrlSafeBase64.encode64(header.to_json),
-          payload:   UrlSafeBase64.encode64(self.to_json),
-          signature: UrlSafeBase64.encode64(signature.to_s)
+          protected: Base64.urlsafe_encode64(header.to_json, padding: false),
+          payload:   Base64.urlsafe_encode64(self.to_json, padding: false),
+          signature: Base64.urlsafe_encode64(signature.to_s, padding: false)
         }
       else
         super
       end
     end
 
+    def pretty_generate
+      [
+        JSON.pretty_generate(header),
+        JSON.pretty_generate(self)
+      ]
+    end
+
     class << self
-      def decode_compact_serialized(jwt_string, key_or_secret)
+      def decode_compact_serialized(jwt_string, key_or_secret, algorithms = nil, encryption_methods = nil)
         case jwt_string.count('.') + 1
         when JWS::NUM_OF_SEGMENTS
-          JWS.decode_compact_serialized jwt_string, key_or_secret
+          JWS.decode_compact_serialized jwt_string, key_or_secret, algorithms
         when JWE::NUM_OF_SEGMENTS
-          JWE.decode_compact_serialized jwt_string, key_or_secret
+          JWE.decode_compact_serialized jwt_string, key_or_secret, algorithms, encryption_methods
         else
           raise InvalidFormat.new("Invalid JWT Format. JWT should include #{JWS::NUM_OF_SEGMENTS} or #{JWE::NUM_OF_SEGMENTS} segments.")
         end
       end
 
-      def decode_json_serialized(input, key_or_secret)
+      def decode_json_serialized(input, key_or_secret, algorithms = nil, encryption_methods = nil)
         input = input.with_indifferent_access
         if (input[:signatures] || input[:signature]).present?
-          JWS.decode_json_serialized input, key_or_secret
+          JWS.decode_json_serialized input, key_or_secret, algorithms
         elsif input[:ciphertext].present?
-          JWE.decode_json_serialized input, key_or_secret
+          JWE.decode_json_serialized input, key_or_secret, algorithms, encryption_methods
         else
           raise InvalidFormat.new("Unexpected JOSE JSON Serialization Format.")
         end
       end
+
+      def pretty_generate(jwt_string)
+        decode(jwt_string, :skip_verification).pretty_generate
+      end
     end
   end
 end
@@ -109,4 +112,4 @@ require 'json/jws'
 require 'json/jwe'
 require 'json/jwk'
 require 'json/jwk/jwkizable'
-require 'json/jwk/set'
\ No newline at end of file
+require 'json/jwk/set'
